@@ -49,8 +49,8 @@ void zmq::client_t::xattach_pipe (pipe_t *pipe_, bool subscribe_to_all_)
 
     zmq_assert (pipe_);
 
-    fq.attach (pipe_);
-    lb.attach (pipe_);
+    _fq.attach (pipe_);
+    _lb.attach (pipe_);
 }
 
 int zmq::client_t::xsend (msg_t *msg_)
@@ -60,25 +60,24 @@ int zmq::client_t::xsend (msg_t *msg_)
         errno = EINVAL;
         return -1;
     }
-    return lb.sendpipe (msg_, NULL);
+    return _lb.sendpipe (msg_, NULL);
 }
 
 int zmq::client_t::xrecv (msg_t *msg_)
 {
-    int rc = fq.recvpipe (msg_, NULL);
+    int rc = _fq.recvpipe (msg_, NULL);
 
     // Drop any messages with more flag
     while (rc == 0 && msg_->flags () & msg_t::more) {
-
         // drop all frames of the current multi-frame message
-        rc = fq.recvpipe (msg_, NULL);
+        rc = _fq.recvpipe (msg_, NULL);
 
         while (rc == 0 && msg_->flags () & msg_t::more)
-            rc = fq.recvpipe (msg_, NULL);
+            rc = _fq.recvpipe (msg_, NULL);
 
         // get the new message
         if (rc == 0)
-            rc = fq.recvpipe (msg_, NULL);
+            rc = _fq.recvpipe (msg_, NULL);
     }
 
     return rc;
@@ -86,31 +85,31 @@ int zmq::client_t::xrecv (msg_t *msg_)
 
 bool zmq::client_t::xhas_in ()
 {
-    return fq.has_in ();
+    return _fq.has_in ();
 }
 
 bool zmq::client_t::xhas_out ()
 {
-    return lb.has_out ();
+    return _lb.has_out ();
 }
 
 const zmq::blob_t &zmq::client_t::get_credential () const
 {
-    return fq.get_credential ();
+    return _fq.get_credential ();
 }
 
 void zmq::client_t::xread_activated (pipe_t *pipe_)
 {
-    fq.activated (pipe_);
+    _fq.activated (pipe_);
 }
 
 void zmq::client_t::xwrite_activated (pipe_t *pipe_)
 {
-    lb.activated (pipe_);
+    _lb.activated (pipe_);
 }
 
 void zmq::client_t::xpipe_terminated (pipe_t *pipe_)
 {
-    fq.pipe_terminated (pipe_);
-    lb.pipe_terminated (pipe_);
+    _fq.pipe_terminated (pipe_);
+    _lb.pipe_terminated (pipe_);
 }
