@@ -38,14 +38,14 @@
 
 zmq::server_t::server_t (class ctx_t *parent_, uint32_t tid_, int sid_) :
     socket_base_t (parent_, tid_, sid_, true),
-    next_routing_id (generate_random ())
+    _next_routing_id (generate_random ())
 {
     options.type = ZMQ_SERVER;
 }
 
 zmq::server_t::~server_t ()
 {
-    zmq_assert (outpipes.empty ());
+    zmq_assert (_out_pipes.empty ());
 }
 
 void zmq::server_t::xattach_pipe (pipe_t *pipe_, bool subscribe_to_all_)
@@ -54,40 +54,41 @@ void zmq::server_t::xattach_pipe (pipe_t *pipe_, bool subscribe_to_all_)
 
     zmq_assert (pipe_);
 
-    uint32_t routing_id = next_routing_id++;
+    uint32_t routing_id = _next_routing_id++;
     if (!routing_id)
-        routing_id = next_routing_id++;        //  Never use Routing ID zero
+        routing_id = _next_routing_id++; //  Never use Routing ID zero
 
     pipe_->set_server_socket_routing_id (routing_id);
     //  Add the record into output pipes lookup table
     outpipe_t outpipe = {pipe_, true};
-    bool ok = outpipes.ZMQ_MAP_INSERT_OR_EMPLACE (routing_id, outpipe).second;
+    bool ok = _out_pipes.ZMQ_MAP_INSERT_OR_EMPLACE (routing_id, outpipe).second;
     zmq_assert (ok);
 
-    fq.attach (pipe_);
+    _fq.attach (pipe_);
 }
 
 void zmq::server_t::xpipe_terminated (pipe_t *pipe_)
 {
-    outpipes_t::iterator it = outpipes.find (pipe_->get_server_socket_routing_id ());
-    zmq_assert (it != outpipes.end ());
-    outpipes.erase (it);
-    fq.pipe_terminated (pipe_);
+    out_pipes_t::iterator it =
+      _out_pipes.find (pipe_->get_server_socket_routing_id ());
+    zmq_assert (it != _out_pipes.end ());
+    _out_pipes.erase (it);
+    _fq.pipe_terminated (pipe_);
 }
 
 void zmq::server_t::xread_activated (pipe_t *pipe_)
 {
-    fq.activated (pipe_);
+    _fq.activated (pipe_);
 }
 
 void zmq::server_t::xwrite_activated (pipe_t *pipe_)
 {
-    outpipes_t::iterator it;
-    for (it = outpipes.begin (); it != outpipes.end (); ++it)
+    out_pipes_t::iterator it;
+    for (it = _out_pipes.begin (); it != _out_pipes.end (); ++it)
         if (it->second.pipe == pipe_)
             break;
 
-    zmq_assert (it != outpipes.end ());
+    zmq_assert (it != _out_pipes.end ());
     zmq_assert (!it->second.active);
     it->second.active = true;
 }
@@ -101,16 +102,15 @@ int zmq::server_t::xsend (msg_t *msg_)
     }
     //  Find the pipe associated with the routing stored in the message.
     uint32_t routing_id = msg_->get_routing_id ();
-    outpipes_t::iterator it = outpipes.find (routing_id);
+    out_pipes_t::iterator it = _out_pipes.find (routing_id);
 
-    if (it != outpipes.end ()) {
+    if (it != _out_pipes.end ()) {
         if (!it->second.pipe->check_write ()) {
             it->second.active = false;
             errno = EAGAIN;
             return -1;
         }
-    }
-    else {
+    } else {
         errno = EHOSTUNREACH;
         return -1;
     }
@@ -124,8 +124,7 @@ int zmq::server_t::xsend (msg_t *msg_)
         // Message failed to send - we must close it ourselves.
         rc = msg_->close ();
         errno_assert (rc == 0);
-    }
-    else
+    } else
         it->second.pipe->flush ();
 
     //  Detach the message from the data buffer.
@@ -138,20 +137,19 @@ int zmq::server_t::xsend (msg_t *msg_)
 int zmq::server_t::xrecv (msg_t *msg_)
 {
     pipe_t *pipe = NULL;
-    int rc = fq.recvpipe (msg_, &pipe);
+    int rc = _fq.recvpipe (msg_, &pipe);
 
     // Drop any messages with more flag
     while (rc == 0 && msg_->flags () & msg_t::more) {
-
         // drop all frames of the current multi-frame message
-        rc = fq.recvpipe (msg_, NULL);
+        rc = _fq.recvpipe (msg_, NULL);
 
         while (rc == 0 && msg_->flags () & msg_t::more)
-            rc = fq.recvpipe (msg_, NULL);
+            rc = _fq.recvpipe (msg_, NULL);
 
         // get the new message
         if (rc == 0)
-            rc = fq.recvpipe (msg_, &pipe);
+            rc = _fq.recvpipe (msg_, &pipe);
     }
 
     if (rc != 0)
@@ -167,7 +165,7 @@ int zmq::server_t::xrecv (msg_t *msg_)
 
 bool zmq::server_t::xhas_in ()
 {
-    return fq.has_in ();
+    return _fq.has_in ();
 }
 
 bool zmq::server_t::xhas_out ()
@@ -180,5 +178,5 @@ bool zmq::server_t::xhas_out ()
 
 const zmq::blob_t &zmq::server_t::get_credential () const
 {
-    return fq.get_credential ();
+    return _fq.get_credential ();
 }
